package java.text;

import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.util.Arrays;

public class ChoiceFormat extends NumberFormat
{
    private static final long serialVersionUID = 1795184449645032964L;
    private double[] choiceLimits;
    private String[] choiceFormats;
    static final long SIGN = Long.MIN_VALUE;
    static final long EXPONENT = 9218868437227405312L;
    static final long POSITIVEINFINITY = 9218868437227405312L;
    
    public void applyPattern(String s) {
        StringBuffer[] array = new StringBuffer[2];
        for (int i = 0; i < array.length; ++i) {
            array[i] = new StringBuffer();
        }
        double[] array2 = new double[30];
        String[] array3 = new String[30];
        int n = 0;
        int n2 = 0;
        double n3 = 0.0;
        double n4 = Double.NaN;
        int n5 = 0;
        for (int j = 0; j < s.length(); ++j) {
            int char1 = s.charAt(j);
            if (char1 == '\'') {
                if (j + 1 < s.length() && s.charAt(j + 1) == char1) {
                    array[n2].append((char)char1);
                    ++j;
                }
                else {
                    n5 = (n5 == 0);
                }
            }
            else if (n5 != 0) {
                array[n2].append((char)char1);
            }
            else if (char1 == '<' || char1 == '#' || char1 == '≤') {
                if (array[0].length() == 0) {
                    throw new IllegalArgumentException();
                }
                try {
                    String string = array[0].toString();
                    if (string.equals("∞")) {
                        n3 = Double.POSITIVE_INFINITY;
                    }
                    else if (string.equals("-∞")) {
                        n3 = Double.NEGATIVE_INFINITY;
                    }
                    else {
                        n3 = (double)Double.valueOf(array[0].toString());
                    }
                }
                catch (Exception ex) {
                    throw new IllegalArgumentException();
                }
                if (char1 == '<' && n3 != Double.POSITIVE_INFINITY && n3 != Double.NEGATIVE_INFINITY) {
                    n3 = nextDouble(n3);
                }
                if (n3 <= n4) {
                    throw new IllegalArgumentException();
                }
                array[0].setLength(0);
                n2 = 1;
            }
            else if (char1 == '|') {
                if (n == array2.length) {
                    array2 = doubleArraySize(array2);
                    array3 = this.doubleArraySize(array3);
                }
                array2[n] = n3;
                array3[n] = array[1].toString();
                ++n;
                n4 = n3;
                array[1].setLength(0);
                n2 = 0;
            }
            else {
                array[n2].append((char)char1);
            }
        }
        if (n2 == 1) {
            if (n == array2.length) {
                array2 = doubleArraySize(array2);
                array3 = this.doubleArraySize(array3);
            }
            array2[n] = n3;
            array3[n] = array[1].toString();
            ++n;
        }
        System.arraycopy(array2, 0, this.choiceLimits = new double[n], 0, n);
        System.arraycopy(array3, 0, this.choiceFormats = new String[n], 0, n);
    }
    
    public String toPattern() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < this.choiceLimits.length; ++i) {
            if (i != 0) {
                sb.append('|');
            }
            double previousDouble = previousDouble(this.choiceLimits[i]);
            if (Math.abs(Math.IEEEremainder(this.choiceLimits[i], 1.0)) < Math.abs(Math.IEEEremainder(previousDouble, 1.0))) {
                sb.append("" + this.choiceLimits[i]);
                sb.append('#');
            }
            else {
                if (this.choiceLimits[i] == Double.POSITIVE_INFINITY) {
                    sb.append("∞");
                }
                else if (this.choiceLimits[i] == Double.NEGATIVE_INFINITY) {
                    sb.append("-∞");
                }
                else {
                    sb.append("" + previousDouble);
                }
                sb.append('<');
            }
            String str = this.choiceFormats[i];
            int n = str.indexOf(60) >= 0 || str.indexOf(35) >= 0 || str.indexOf(8804) >= 0 || str.indexOf(124) >= 0;
            if (!(n == 0)) {
                sb.append('\'');
            }
            if (str.indexOf(39) < 0) {
                sb.append(str);
            }
            else {
                for (int j = 0; j < str.length(); ++j) {
                    int char1 = str.charAt(j);
                    sb.append((char)char1);
                    if (char1 == '\'') {
                        sb.append((char)char1);
                    }
                }
            }
            if (!(n == 0)) {
                sb.append('\'');
            }
        }
        return sb.toString();
    }
    
    public ChoiceFormat(String s) {
        this.applyPattern(s);
    }
    
    public ChoiceFormat(double[] array, String[] array2) {
        this.setChoices(array, array2);
    }
    
    public void setChoices(double[] original, String[] original2) {
        if (original.length != original2.length) {
            throw new IllegalArgumentException("Array and limit arrays must be of the same length.");
        }
        this.choiceLimits = Arrays.copyOf(original, original.length);
        this.choiceFormats = (String[])Arrays.copyOf(original2, original2.length);
        return;
    }
    
    public double[] getLimits() {
        return Arrays.copyOf(this.choiceLimits, this.choiceLimits.length);
    }
    
    public Object[] getFormats() {
        return Arrays.copyOf(this.choiceFormats, this.choiceFormats.length);
    }
    
    @Override
    public StringBuffer format(long n, StringBuffer sb, FieldPosition fieldPosition) {
        return this.format((double)n, sb, fieldPosition);
    }
    
    @Override
    public StringBuffer format(double n, StringBuffer sb, FieldPosition fieldPosition) {
        int n2;
        for (n2 = 0; n2 < this.choiceLimits.length && n >= this.choiceLimits[n2]; ++n2) {}
        if (--n2 < 0) {
            n2 = 0;
        }
        return sb.append(this.choiceFormats[n2]);
    }
    
    @Override
    public Number parse(String s, ParsePosition parsePosition) {
        int errorIndex = parsePosition.index;
        double value = Double.NaN;
        for (int i = 0; i < this.choiceFormats.length; ++i) {
            String other = this.choiceFormats[i];
            if (s.regionMatches(errorIndex, other, 0, other.length())) {
                parsePosition.index = errorIndex + other.length();
                double n = this.choiceLimits[i];
                if (parsePosition.index > errorIndex) {
                    errorIndex = parsePosition.index;
                    value = n;
                    if (errorIndex == s.length()) {
                        break;
                    }
                }
            }
        }
        if (parsePosition.index == (parsePosition.index = errorIndex)) {
            parsePosition.errorIndex = errorIndex;
        }
        return new Double(value);
    }
    
    public static final double nextDouble(double n) {
        return nextDouble(n, true);
    }
    
    public static final double previousDouble(double n) {
        return nextDouble(n, false);
    }
    
    @Override
    public Object clone() {
        ChoiceFormat choiceFormat = (ChoiceFormat)super.clone();
        choiceFormat.choiceLimits = (double[])this.choiceLimits.clone();
        choiceFormat.choiceFormats = (String[])this.choiceFormats.clone();
        return choiceFormat;
    }
    
    @Override
    public int hashCode() {
        int length = this.choiceLimits.length;
        if (this.choiceFormats.length > 0) {
            length ^= this.choiceFormats[this.choiceFormats.length - 1].hashCode();
        }
        return length;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null) {
            return false;
        }
        if (this == o) {
            return true;
        }
        if (this.getClass() != o.getClass()) {
            return false;
        }
        ChoiceFormat choiceFormat = (ChoiceFormat)o;
        return Arrays.equals(this.choiceLimits, choiceFormat.choiceLimits) && Arrays.equals(this.choiceFormats, choiceFormat.choiceFormats);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.choiceLimits.length != this.choiceFormats.length) {
            throw new InvalidObjectException("limits and format arrays of different length.");
        }
        return;
    }
    
    public static double nextDouble(double n, boolean b) {
        if (Double.isNaN(n)) {
            return n;
        }
        if (n == 0.0) {
            double longBitsToDouble = Double.longBitsToDouble(1L);
            if (b) {
                return longBitsToDouble;
            }
            return -longBitsToDouble;
        }
        else {
            long doubleToLongBits = Double.doubleToLongBits(n);
            long n2 = doubleToLongBits & Long.MAX_VALUE;
            if (doubleToLongBits > 0L == b) {
                if (n2 != 9218868437227405312L) {
                    ++n2;
                }
            }
            else {
                --n2;
            }
            return Double.longBitsToDouble(n2 | (doubleToLongBits & Long.MIN_VALUE));
        }
    }
    
    private static double[] doubleArraySize(double[] array) {
        int length = array.length;
        double[] array2 = new double[length * 2];
        System.arraycopy(array, 0, array2, 0, length);
        return array2;
    }
    
    private String[] doubleArraySize(String[] array) {
        int length = array.length;
        String[] array2 = new String[length * 2];
        System.arraycopy(array, 0, array2, 0, length);
        return array2;
    }
}
